/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef MHDC_h
#define MHDC_h

#include <MH.h>
#include <MHVector.h>

class MHChart;
class MHPoint;
class MHChartSettings;



/**
*  A struct to store data for each MHChart object
*/
class MHChartArea : public MH {
public:
    double                  aW;
    double                  aH;
    double                  aX;
    double                  aY;
    MHChart*                aChart;
};



/**
*  Inherit from this object to implement REAL device context functions. Either for
*  a gui toolkit or an image of some kind.
*/
class MHDC : public MH {
public:
                            MHDC ();
    virtual                 ~MHDC ();

    void                    Add (MHChart* pChart, double x, double y, double w, double h);
    void                    Align (int align);
    virtual void            BeginClip (int x1, int y1, int x2, int y2) {;}
    const char*             Class () {return "MHDC";}
    void                    Clear ();
    void                    CreateTimeSerie( );
    void                    Debug ();
    MHChart*                GetChart (int id);
    int                     GetFontSize () {return aFontSize;}
    int                     GetLargestDataSet ();
    MHChartSettings*        GetSettings (int id);
    void                    Paint ();
    virtual bool            QueryLeft (int x, int y, MHPoint* p);
    virtual bool            QueryRight (int x, int y, MHPoint* p);
    virtual void            Recalc (bool bInit, int w = -1, int h = -1);
    void                    SetFontSize (int size) {aFontSize = size;}
    void                    SetStartInDataSet (int start);

    virtual void            Color (int color) = 0;
    virtual void            DrawLine (int x1, int y1, int x2, int y2) = 0;
    virtual void            DrawRect (int x1, int y1, int x2, int y2) = 0;
    virtual void            DrawText (const char* p, int x, int y){;}
    virtual void            EndClip () {;}
    virtual int             GetCharHeight() = 0;
    virtual int             GetHeight () = 0;
    virtual int             GetStringWidth (const char* pString) = 0;
    virtual int             GetWidth () = 0;
    virtual void            SetBrushColor (int Color) = 0;
    virtual void            SetPenColor (int Color) = 0;
    virtual void            SetScrollPos (int Pos){;}
    virtual void            SetScrollRange (){;}

protected:
    int                     aW;
    int                     aH;
    int                     aFontSize;
    MHVector*               aChartAreas;
    MHVector                aDates;
};

#endif
