/**
*  @version 1999-07-02
*/

#include <MHList.h>



/**
* Create node
*/
MHLNode::MHLNode () {
    itsObject = 0;
    itsNext   = itsPrev = 0;
}



/**
* Create node
*/
MHLNode::MHLNode (MH* o) {
    itsObject = o;
    itsNext   = itsPrev = 0;
}



/**
*  Swap two nodes
*  @param MHLNode* - First node
*  @param MHLNode* - Second node
*/
void MHLNode::Swap (MHLNode *p1, MHLNode *p2) {
    MH* o;

    o = p1->itsObject;
    p1->itsObject = p2->itsObject;
    p2->itsObject = o;
}



/**
*  Constructor
*/
MHList::MHList () : MH() {
    itsCount = 0;
    itsHead  = itsTail = itsCurrent = 0;
}



/**
*  Remove all objects from memory
*/
MHList::~MHList () {
    Empty ();
}




/**
*  Remove all objects in list
*/
void MHList::empty (bool erase) {
    MHLNode* tmp;

    while (itsHead != 0) {
        tmp = itsHead->itsNext;
        if (erase) delete itsHead->itsObject;
        delete itsHead;
        itsHead = tmp;
    }

    itsHead = itsTail = itsCurrent = 0;
    itsCount = 0;
}






/**
*  Find object in list and set current object to the found object
*  @return bool - true if found
*/
MH* MHList::Find (MH* o) {
    MHLNode* tmp = itsHead;

    //  Search through the list and break if the right key is found
    while (tmp != 0) {
        if (tmp->itsObject && tmp->itsObject->Compare(o, 0) == 0) {
            itsCurrent = tmp;
            return itsCurrent->itsObject;
        }
        tmp = tmp->itsNext;
    }
    return 0;
}




/**
*  Insert a object before current object
*  @param MH* - Object to add
*/
void MHList::Insert (MH* o) {
    MHLNode* tmp = itsHead;

    if (tmp) {
        MHLNode* node = new MHLNode (o);
        ASS(node);

        node->itsNext = itsHead;
        node->itsPrev = 0;

        if (itsHead)
            itsHead->itsPrev = node;

        itsHead = node;
        itsCount++;
    }
    else
        Push (o);
}



/**
*  Set current object to next object
*  @return MH* - Object or 0
*/
MH* MHList::Next () {
    if (itsCurrent)
        itsCurrent = itsCurrent->itsNext;

    return (itsCurrent) ? itsCurrent->itsObject : 0;
}



/**
*  Set current object to previous object
*  @return MH* - Object or 0
*/
MH* MHList::Prev () {
    if (itsCurrent)
        itsCurrent = itsCurrent->itsPrev;

    return (itsCurrent) ? itsCurrent->itsObject : 0;
}



/**
*  Append object to list
*  @param MH*   -  Object to add
*/
void MHList::Push (MH* o) {
    MHLNode* node = new MHLNode (o);
    ASS(node)

    if (!itsHead) {
        itsHead = itsTail = itsCurrent = node;
    }
    else {
        itsTail->itsNext = node;
        itsTail->itsNext->itsPrev = itsTail;
        itsTail = itsTail->itsNext;
    }
    itsCount++;
}



/**
*  Remove current object from list
*  @return MH* - Return removed object, object is NOT deleted
*/
MH* MHList::Remove () {
    MH*      o = 0;
    MHLNode* tmp;

    //  Move links in the linked list so it's not broken
    if (itsCurrent) {
        if (itsCurrent->itsPrev)
            itsCurrent->itsPrev->itsNext = itsCurrent->itsNext;
        else
            itsHead = itsCurrent->itsNext;

        if (itsCurrent->itsNext) {
            itsCurrent->itsNext->itsPrev = itsCurrent->itsPrev;
            tmp = itsCurrent->itsNext;
        }
        else
            itsTail = tmp = itsCurrent->itsPrev;

        o = itsCurrent->itsObject;
        delete itsCurrent;
        itsCount--;
        itsCurrent = tmp;
    }
    return o;
}



/**
*  Sort list
*  @param bool - true to ascending
*/
void MHList::Sort (bool ascending) {
    MHLNode  *i = itsHead;
    MHLNode  *j;

    while (i && i->itsNext) {
        j = i->itsNext;
        while (j) {
            if (ascending == true) {
                if (i->itsObject->Compare(j->itsObject, 0) > 0)
                    MHLNode::Swap (i, j);
            }
            else {
                if (i->itsObject->Compare(j->itsObject, 0) < 0)
                    MHLNode::Swap (i, j);
            }
            j = j->itsNext;
        }
        i = i->itsNext;
    }
}
