#include <MHVector.h>

int MHVector::aaSortVectorCol = 0;



/**
*  Create a new vector
*/
int MHVector::Compare (const MH* o, int type) {
    MHVector*   ve  = (MHVector*) o;
    MH*         mh1 = At (MHVector::aaSortVectorCol);
    MH*         mh2 = ve->At (MHVector::aaSortVectorCol);

    if (mh1 && mh2)
        return mh1->Compare (mh2, type);
    return -1;
}



/**
*  Create a new vector
*/
void MHVector::create (int capacity) {
    aSize        = 0;
    aCapacity    = capacity;
    aVector      = new MH*[aCapacity];
    aSortType    = 0;
}



/**
*  Empty vector but don't delete objects.
*/
void MHVector::Empty () {
    if (aCapacity) delete []aVector;
    aSize     = 0;
    aCapacity = 0;
    aVector   = 0;
}



/**
*  Erase vector and delete all objects.
*/
void MHVector::Erase () {
    for (int f = 0; f < aSize; f++)
        delete aVector[f];
    delete []aVector;
    aSize     = 0;
    aCapacity = 0;
    aVector   = 0;
}



/**
*  Erase object in vector
*  @param int  - Index in vector
*/
void MHVector::Erase (int index) {
    MH* node = Pop (index);
    delete node;
}



/**
*  Erase object in vector
*  @param MH - Object to erase
*/
void MHVector::Erase (MH* o) {
    MH* node = Pop (o);
    delete node;
}



/**
*  Find a object
*  @param MH*   - Object to use for searching
*  @param int&  - Start index to go from
*  @param int   - Compare type
*  @return MH*  - Object if found else 0
*/
MH* MHVector::Find (const MH* o, int& mid, int type) {
    int  low  = 0;
    int  high = aSize;
    int  comp = 0;

    if (type != aSortType) Sort (type);
    mid = 0;
    if (aSize == 0) return false;

    while (low != high) {
        mid  = low + (high - low) / 2;
        comp = aVector[mid]->Compare(o, aSortType) ;

        if (comp == 0)
            return aVector[mid];
        else if (comp > 0)
            high = mid;
        else
            low = mid + 1;
    }

    if (comp < 0) mid++;
    return 0;
}



/**
*  Find object or return first object after the searched one
*  @param MH* - Object with key to search for
*  @param int - Compare type
*/
MH* MHVector::FindAfter (const MH* o, int type) {
    int i;

    if (Find(o, i, type))
        return At(i);

    if (Size() > 0 && i < Size())
        return At(i);
    else
        return 0;
}



/**
*  Find object or return first object before searched one
*/
MH* MHVector::FindBefore (const MH* o, int type) {
    int i;

    if (Find(o, i, type))
        return At(i);

    if (i > 0 && i <= Size())
        return At(i - 1);
    else
        return 0;
}



/**
*  Insert object first in list
*  @param MH* - Object to insert
*  @param int - Position in vector, default 0
*/
void MHVector::Insert (MH* node, int pos) {
    int f;

    if (pos < 0) return;
    if (aSize == aCapacity) {
        //  Increase capacity now
        aCapacity = (aCapacity == 0) ? 10 : aCapacity * 2;
        MH** tmp = new MH*[aCapacity];
        for (f = aSize - 1; f >= pos; f--)
            tmp[f + 1] = aVector[f];
        for (f = 0; f < pos; f++)
            tmp[f] = aVector[f];
        tmp[pos] = node;
        delete []aVector;
        aVector = tmp;
        aSize++;
    }
    else if (pos >= aSize)
        Push (node);
    else {
        for (f = aSize - 1; f >= pos; f--)
            aVector[f + 1] = aVector[f];
        aVector[pos] = node;
        aSize++;
    }
}



/**
*  Insert object at the first slot where key is larger
*  @param MH* - Object to insert
*  @param int - Compare type
*/
void MHVector::InsertSorted (MH* node, int type) {
    int f;

    if (aSize == aCapacity) {
        //  Increase capacity now
        aCapacity = (aCapacity == 0) ? 10 : aCapacity * 2;
        MH** tmp = new MH*[aCapacity];

        for (f = aSize - 1; f >= 0; f--) {
            if (aVector[f]->Compare (node, type) < 1) break;
            tmp[f + 1] = aVector[f];
        }
        tmp[f + 1] = node;
        for (; f >= 0; f--)
            tmp[f] = aVector[f];

        delete []aVector;
        aVector = tmp;
        aSize++;
    }
    else {
        for (f = aSize - 1; f >= 0; f--) {
            if (aVector[f]->Compare (node, type) < 1) break;
            aVector[f + 1] = aVector[f];
        }
        aVector[f + 1] = node;
        aSize++;
    }
}



/**
*
*/
void MHVector::insertionSort (MH** ve, int lo0, int hi0) {
    int i;
    int j;
    MH* v;

    for (i = lo0 + 1; i <= hi0; i++) {
        v = ve[i];
        j = i;
        while ((j > lo0) && (ve[j - 1]->Compare(v, aSortType) > 0)) {
            ve[j] = ve[j - 1];
            j--;
        }
        ve[j] = v;
    }
}



/**
*
*/
void MHVector::heapSort (MH** ve, int size) {
    int i;
    MH* tmp;

    for (i = (size / 2) - 1; i >= 0; i--)
        MHVector::siftDown (aSortType, ve, i, size - 1);

    for (i = size - 1; i >= 1; i--) {
        tmp = ve[0];
        ve[0] = ve[i];
        ve[i] = tmp;
        MHVector::siftDown (aSortType, ve, 0, i - 1);
  }
}



/**
*  Find a object. Search from start index. Objects must be sorted.
*  @param MH*   - Object to use for searching
*  @param int   - Start index
*  @param int   - Compare type
*  @param int*  - The index for the found number
*  @return MH*  - Found object or 0 if not found
*/
MH* MHVector::Search (const MH* o, int startIndex, int type, int* foundIndex) {
    int cmp;

    for (; startIndex < aSize; startIndex++) {
        cmp = aVector[startIndex]->Compare (o, type);
        
        if (cmp == 0) {
            if (foundIndex) *foundIndex = startIndex;
            return aVector[startIndex];
        }
        else if (cmp > 0)
            return 0;
    }
    return 0;
}



/**
*  Find a object. All objects are searched until found or no more.
*  @param MH*   - Object to use for searching
*  @param int   - Start index
*  @param int   - Compare type
*  @return MH*  - Found object or 0 if not found
*/
MH* MHVector::SearchAll (const MH* o, int startIndex, int type, int* foundIndex) {
    int cmp;

    for (; startIndex < aSize; startIndex++) {
        cmp = aVector[startIndex]->Compare (o, type);
        
        if (cmp == 0) {
            if (foundIndex) *foundIndex = startIndex;
            return aVector[startIndex];
        }
    }
    return 0;
}



/**
*
*/
void MHVector::siftDown (int type, MH** ve, int root, int bottom) {
    int  done = 0;
    int  maxChild;
    MH*  tmp;

    done = 0;
    while ((root * 2) <= bottom && !done) {
        if (root * 2 == bottom)
            maxChild = root * 2;
        else if (ve[root * 2]->Compare (ve[root * 2 + 1], type) > 0)
            maxChild = root * 2;
        else
            maxChild = root * 2 + 1;

        if (ve[root]->Compare (ve[maxChild], type) < 0) {
            tmp = ve[root];
            ve[root] = ve[maxChild];
            ve[maxChild] = tmp;
            root = maxChild;
        }
        else
            done = 1;
    }
}



/**
*  Remove object in vector
*  @param int  - Index in vector
*  @return MH* - Object in vector or 0
*/
MH* MHVector::Pop (int index) {
    if (index < 0 || index >= aSize) return 0;

    MH* node = aVector[index];

    for (int f = index + 1; f < aSize; f++)
        aVector[f - 1] = aVector[f];
    aSize--;
    return node;
}



/**
*  Remove object from vector, only the adress of the object is compared.
*  @param MH*   - Object to find
*  @return MH*  - Found object or 0
*/
MH* MHVector::Pop (MH* o) {
    for (int f = 0; f < aSize; f++) {
        if (aVector[f] == o)
            return Pop (f);
    }
    return 0;
}



/**
*  Push a node to the end of vector.
*  If no more room increase size by the double
*/
void MHVector::Push (MH* node) {
    if (aSize == aCapacity) {
        //  Increase capacity now
        aCapacity = (aCapacity == 0) ? 10 : aCapacity * 2;
        MH** tmp = new MH*[aCapacity];
        for (int f = 0; f < aSize; f++)
            tmp[f] = aVector[f];
        delete []aVector;
        aVector = tmp;
    }
    aVector[aSize] = node;
    aSize++;
}



/**
*
*/
void MHVector::quickSort (int type, MH** ve, int l, int r) {
    int M = 4;
    int i;
    int j;
    MH* v;

    if ((r - l) > M) {
        i = (r + l) / 2;
        if (ve[l]->Compare(ve[i], type) > 0) MHVector::swap (ve, l, i);
        if (ve[l]->Compare(ve[r], type) > 0) MHVector::swap (ve, l, r);
        if (ve[i]->Compare(ve[r], type) > 0) MHVector::swap (ve, i, r);

        j = r - 1;
        MHVector::swap (ve, i, j);
        i = l;
        v = ve[j];

        while (1) {
            while (ve[++i]->Compare(v, type) < 0) {;}
            while (ve[--j]->Compare(v, type) > 0) {;}
            if (j < i) break;
            swap (ve, i, j);
        }
        MHVector::swap (ve, i, r - 1);
        MHVector::quickSort (type, ve, l, j);
        MHVector::quickSort (type, ve, i + 1, r);
    }
}



/**
*  Resize capacity
*  @param int - The number of cells to add to array
*/
void MHVector::Resize (int size) {
    ASS (size > 0)
    if (aCapacity >= size) return;
    aCapacity = size;
    MH** tmp = new MH*[aCapacity];
    for (int f = 0; f < aSize; f++)
        tmp[f] = aVector[f];
    delete []aVector;
    aVector = tmp;
}



/**
*  Reverse order
*/
void MHVector::Reverse () {
    int i = aSize / 2;
    int j = aSize;
    for (int f = 0; f < i; f++)
        MHVector::swap (aVector, f, --j);
}



/**
*
*/
void MHVector::swap (MH** ve, int a, int b) {
    MH* tmp  = ve[a];
    ve[a] = ve[b];
    ve[b] = tmp;
}
