/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "chart/BaseChart.h"
#include <MHDate.h>
#include <MHDateChart.h>
#include <MHDC.h>



/**
*
*/
BaseChart::BaseChart (MHDC* dc, int view, MHDate::DAYS timeType) {
    MHDate start ("19700101");
    MHDate stop;

    aView       = view;
    aDC         = dc;
    aData       = 0;
    aLayout     = (XTRADER_CHART_LAYOUT) 0;
    aLineWidth  = 0;
    aBGColor    = 0;
    aTimeType   = timeType;
    aStartDate  = start.Get (MHDate::LONG_DATE);
    aStopDate   = stop.Get (MHDate::LONG_DATE);
}



/**
*  Check date, swap dates if start date is greater than stop date
*/
void BaseChart::CheckDate () {
    if (aStartDate > aStopDate) {
        MHString tmp = aStartDate;
        aStartDate   = aStopDate;
        aStopDate    = tmp;
    }
}




/**
*  Copy data from cache series to local vector.
*  Convert time serie if settings is set to week time serie.
*  Clip data also if outside user date range.
*/
void BaseChart::copy (MHVector* v1, MHVector* v2, MHVector* ve1, MHVector* ve2) {
    if (aTimeType ==  MHDate::FRIDAY) {
        MHVector* tmp = new MHVector ();
        MHPoint::Day2Week (v1, tmp, MHDate::FRIDAY);
        MHPoint::Copy (tmp, ve1, aStartDate(), aStopDate());
        if (v2) MHPoint::Copy (v2, ve2, aStartDate(), aStopDate());
        tmp->Erase ();
        delete tmp;
    }
    else {
        MHPoint::Copy (v1, ve1, aStartDate(), aStopDate());
        MHPoint::Copy (v2, ve2, aStartDate(), aStopDate());
    }
}



/**
*  Create new chart
*  @return MHDateChart* - The new chart, configured and ready
*  @return bool         - true to show labels
*/
MHDateChart* BaseChart::createChart (bool label) {
    MHDateChart*     c = new MHDateChart ();
    MHChartSettings* s = c->GetSettings ();
    
    s->aShowAll = 0;
    if (label) s->aXLabels = 1;
    s->aYLabels = 1;
    s->aXWidth = aLineWidth;
    s->aTimeType = aTimeType;
    return c;
}


/**
*  Create new chart view
*  @param MHDC& - The device context
*/
void BaseChart::createView () {
    aDC->Clear ();
    aDC->Color (aBGColor);

    switch (aLayout) {
        case XTRADER_SINGLE_LAYOUT:
            aDC->Add (createChart(true), 0, 0, 1, 1);
            break;

        case XTRADER_DOUBLE_LAYOUT:
            aDC->Add (createChart(false), 0.0, 0.0, 1.0, 0.5);
            aDC->Add (createChart(true), 0.0, 0.5, 1.0, 1);
            break;

        case XTRADER_DOUBLE2_LAYOUT:
            aDC->Add (createChart(false), 0.0, 0.0, 1.0, 0.66);
            aDC->Add (createChart(true), 0.0, 0.66, 1.0, 1);
            break;

        case XTRADER_TRIPLE_LAYOUT:
            aDC->Add (createChart(false), 0.0, 0.0, 1.0, 0.33);
            aDC->Add (createChart(false), 0.0, 0.33, 1.0, 0.66);
            aDC->Add (createChart(true), 0.0, 0.66, 1.0, 1);
            break;

        case XTRADER_TRIPLE2_LAYOUT:
            aDC->Add (createChart(false), 0.0, 0.0, 1.0, 0.5);
            aDC->Add (createChart(false), 0.0, 0.5, 1.0, 0.75);
            aDC->Add (createChart(true), 0.0, 0.75, 1.0, 1);
            break;

        default:
            aDC->Add (createChart(true), 0, 0, 1, 1);
            break;
    }
}



/**
*  Return current chart to be used
*  @param WINDOWS     - What chart area
*/
void BaseChart::getChart (XTRADER_CHART_WINDOW win) {
    aCurrChart  = 0;

   switch (win) {
       case XTRADER_WINDOW_TOP:
            aCurrAlign = true;
            aCurrChart = (MHDateChart*) (aDC->GetChart (1));
            break;

        case XTRADER_WINDOW_TOP2:
            aCurrAlign = false;
            aCurrChart = (MHDateChart*) (aDC->GetChart (1));
            break;

        case XTRADER_WINDOW_MIDDLE:
            aCurrAlign = true;
            aCurrChart = (MHDateChart*) (MHDateChart*) (aDC->GetChart (2));
            break;

        case XTRADER_WINDOW_MIDDLE2:
            aCurrAlign = false;
            aCurrChart = (MHDateChart*) (aDC->GetChart (2));
            break;

        case XTRADER_WINDOW_BOTTOM:
            aCurrAlign = true;
            aCurrChart = (MHDateChart*) (aDC->GetChart (3));
            break;

        case XTRADER_WINDOW_BOTTOM2:
            aCurrAlign = false;
            aCurrChart = (MHDateChart*) (aDC->GetChart (3));
            break;
    }
}
