/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <download/BaseDownload.h>
#include <MHUtil.h>
#include <db/DBFactory.h>
#include <pinet.h>

USING_PTYPES


ipstream* aStream = 0;
mutex* aMutex = 0;


/**
*
*/
BaseDownload::BaseDownload () : MH() {
    aNetError = 0; 
    aStream   = 0;
    aMutex    = 0;
    aBuffer   = new char[BUFFER_SIZE + 2];
    aMutex    = new mutex ();
}



/**
*
*/
BaseDownload::~BaseDownload () {
    delete aBuffer;
    delete aMutex;
    aMutex = 0;
}

    

/**
*
*/
void BaseDownload::Close () {
    aMutex->lock ();
    if (aStream) {
        try {
            aStream->cancel();
        }
        catch (estream* e) {
            delete e;
        }
    }
    aMutex->unlock ();
}



/**
*  !!! What to do with blocking requests?
*/
const char* BaseDownload::getData (const char* server, const char* url) {
    try {
        ipaddress adress = phostbyname (server);
        aBuffer[0] = '\0';

        if (adress == ipnone) {
            DBFactory::WriteLog ("Error contacting %s%s", server, url);
            aNetError++;
        }
        else {
            char tmp[256];
            aStream = new ipstream (adress, 80);

            SNPRINTF (tmp, 255, "GET %s", url);
            aStream->open();
            aStream->putline(tmp);
            aStream->flush();
            if (aStream->waitfor(5000) == true) {
                int index = 0;
                while (!aStream->get_eof() && index < BUFFER_SIZE) {
                    aBuffer[index++] = aStream->get();
                }
                aBuffer[index++] = '\0';
            }
            aStream->close();
            ipstream* s = aStream;
            aStream = 0;
            delete s;
        }
    }
    catch (estream* e) {
        DBFactory::WriteLog ("Error reading from %s%s (%s)", server, url, pconst(e->get_message()));
        delete e;

        try {
            if (aStream) {
                aStream->close();
                delete aStream;
            }
        }
        catch (estream* e) {
            delete e;
        }
        aNetError++;
        aBuffer[0] = '\0';
    }
    aStream = 0;
    return aBuffer;
}
