/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <download/YahooDownload.h>
#include <db/DBFactory.h>
#include <db/Price.h>
#include <db/Security.h>
#include <MHDate.h>
#include <MHUtil.h>
#include <MHVector.h>
#include <stdio.h>



/**
*
*/
YahooDownload::YahooDownload () :
BaseDownload() {
}



/**
*
*/
YahooDownload::~YahooDownload () {
}



/**
*
*/
MHVector* YahooDownload::Load (BaseDownload::TYPE type, const char* start, const char* stop, Security* sec) {
    MHVector*   ve = 0;

    switch (type) {
        case LATEST: {
            ve = new MHVector();
            SNPRINTF (aUrl, 255, "finance.yahoo.com");
            SNPRINTF (aLink, 255, "/d/quotes.csv?s=%s&f=snl1d1t1c1ohgv&e=.csv", sec->GetCode()->Get());
            const char* data    = getData(aUrl, aLink);
            Price*      price   = parseLatest (sec, data);

            if (price)
                ve->Push (price);
            break;
        }

        case HISTORY: {
            MHDate start2 (start);
            MHDate stop2 (stop);
            SNPRINTF (aUrl, 255, "table.finance.yahoo.com");
            SNPRINTF (aLink, 255, "/table.csv?s=%s&d=%d&e=%d&f=%d&g=d&a=%d&b=%d&c=%d&ignore=.csv", 
                sec->GetCode()->Get(), stop2.GetMonth() - 1, stop2.GetDate(), stop2.GetLongYear(), start2.GetMonth() - 1, start2.GetDate(), start2.GetLongYear());
            const char* data = getData(aUrl, aLink);
            ve = parseHistory (sec, data);
            break;
        }
    }

    return ve;
}




/**
*  Parse text from yahoo and return a Price object if ok else 0
*/
Price* YahooDownload::parseLatest (Security* sec, const char* data) {
    Price*      price   = 0;
    MHString    s       = data;

    s.Remove ('"');
    s.Strip ();

    if (data && *data) {
        MHVector ve;

        s.Split (',', ve);

        if (ve.Size() == 10) {
            MHString* s = (MHString*) ve[0];
            s->ToLower();

            if (*s == sec->GetCode()->Get()) {
                MHString*   sda   = (MHString*) ve[3];
                MHString*   shi   = (MHString*) ve[7];
                MHString*   slo   = (MHString*) ve[8];
                MHString*   scl   = (MHString*) ve[2];
                MHString*   sop   = (MHString*) ve[6];
                MHString*   svo   = (MHString*) ve[9];
                double      hi    = shi->ToDouble();
                double      lo    = slo->ToDouble();
                double      cl    = scl->ToDouble();
                double      op    = sop->ToDouble();
                double      vo    = svo->ToDouble();
                MHString*   date  = MHDate::ParseString (sda);

                if (date->Size() == 8 && MHDate::ValidDate(date->Get()) == true) {
                    price = new Price(date->Get(), Price::PRICE, hi, lo, cl, op, vo);
                    price->Validate ();
                }
                delete date;
            }
        }
        ve.Erase();
    }

    if (!price) {
        DBFactory::WriteLog ("Can't parse latest data from %s%s", aUrl, aLink);
        if (data && *data)
            DBFactory::WriteLog ("Response from server is %s", data);
    }
    return price;
}



/**
*  Parse text from yahoo and return a Price object if ok else 0
*/
MHVector* YahooDownload::parseHistory (Security* sec, const char* data) {
    MHVector* rows = new MHVector ();

    if (data && *data) {
        MHString s;
        MHVector ve;

        s = data;
        s.Split ('\n', ve);
        for (int f = 0; f < ve.Size(); f++) {
            MHString* s = (MHString*) ve[f];
            MHVector  ve2;

            s->Strip();
            s->Split (',', ve2);

            if (ve2.Size() == 7) {
                MHString*   sda   = (MHString*) ve2[0];
                MHString*   shi   = (MHString*) ve2[2];
                MHString*   slo   = (MHString*) ve2[3];
                MHString*   scl   = (MHString*) ve2[4];
                MHString*   sop   = (MHString*) ve2[1];
                MHString*   svo   = (MHString*) ve2[5];
                double      hi    = shi->ToDouble();
                double      lo    = slo->ToDouble();
                double      cl    = scl->ToDouble();
                double      op    = sop->ToDouble();
                double      vo    = svo->ToDouble();
                MHString*   date  = MHDate::ParseString (sda);

                if (date->Size() == 8 && MHDate::ValidDate(date->Get()) == true) {
                    Price* price = new Price(date->Get(), Price::PRICE, hi, lo, cl, op, vo);
                    price->Validate ();
                    rows->Push (price);
                }
                delete date;
            }
            else if (ve2.Size() > 1) {
                DBFactory::WriteLog ("Can't parse history data from %s%s", aUrl, aLink);
                DBFactory::WriteLog ("Response from server is %s", data);
            }
            ve2.Erase();
        }
        ve.Erase();
    }
    return rows;
}
