/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/PrefPortfolioChartData.h>
#include <db/DBFactory.h>
#include <db/Pref.h>
#include <db/Security.h>
#include <chart/PortfolioChart.h>
#include <Enums.h>
#include <Resource.h>
#include <MHPoint.h>
#include <FL/Fl.h>
#include <math.h>



/**
*  Edit portfolio chart options
*/
PrefPortfolioChartData::PrefPortfolioChartData (int view) :
BaseData () {
    int f;

    PortfolioChart p (0, -1);
    ReportTypes*   types = p.Choice ();

    aView           = view;
    aModels         = types->aNames.Size();
    aChoiceLayout   = MHUtil::Allocate (5, 40);
    aChoiceModels   = MHUtil::Allocate (aModels, 30);
    aChoiceSecurity = 0;
    aVectorSecurity = new MHVector ();

    STRNCPY (aChoiceLayout[0], STRING(LAYOUT_1), 29)
    STRNCPY (aChoiceLayout[1], STRING(LAYOUT_2EQUAL), 29)
    STRNCPY (aChoiceLayout[2], STRING(LAYOUT_2), 29)
    STRNCPY (aChoiceLayout[3], STRING(LAYOUT_3EQUAL), 29)
    STRNCPY (aChoiceLayout[4], STRING(LAYOUT_3), 29)

    for (f = 0; f < aModels; f++) {
        MHString* s = (MHString*) types->aNames[f];
        STRNCPY (aChoiceModels[f], s->Get(), 29)
    }
    reload ("Security");
}



/**
*
*/
PrefPortfolioChartData::~PrefPortfolioChartData () {
    MHUtil::Delete (aChoiceLayout, 5);
    MHUtil::Delete (aChoiceModels,  aModels);
    if (aChoiceSecurity) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);

    aVectorSecurity->Erase ();
    delete aVectorSecurity;
}



/**
*
*/
Fl_Align PrefPortfolioChartData::align (int row, int col) {
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** PrefPortfolioChartData::choice (int row, int col, int& size, int& selected, char* label) {
    size = selected = 0;

    if (row == 0) {
        SNPRINTF (aBuffer, 99, "%s_%d", STRING(PREF_CHARTP_LAYOUT), aView);
        size     = 5;
        selected = DBFactory::GetPrefIntValue (aBuffer, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT);
        return (const char**) aChoiceLayout;
    }
    else if (row >= 1 && row <= 6) {
        setWindowKey (row, 0);
        size     = aModels;
        selected = DBFactory::GetPrefIntValue (aBuffer, 0, aModels, 0);
        return (const char**) aChoiceModels;
    }
    else if (row == 9) {
        int sec = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0);

        size     = aVectorSecurity->Size();
        selected = 0;

        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            if (s->GetID() == sec) {
                selected = f;
                break;
            }
        }
        strncpy (label, STRING(SELECT_SECURITY), 99);
        return (const char**) aChoiceSecurity;
    }
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR PrefPortfolioChartData::editor_type (int row, int col, bool force_custom) {
    if (col == 1) {
        switch (row) {
            case 7:
                return FL_INTEGER_EDITOR;

            case 8:
                return FL_DLG_COLOR_EDITOR;

            case 9:
                return FL_DLG_LIST_EDITOR;

            default:
                return FL_LIST_EDITOR;
        }
    }
    return (FL_TABLE_EDITOR)0;
}



/**
*
*/
void PrefPortfolioChartData::reload (const char* message) {
    if (strcmp (message, "Security") == 0) {
        if (aChoiceSecurity) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);
        aVectorSecurity->Erase ();
        Security::Load (aVectorSecurity);
        aVectorSecurity->Sort (Security::SORT_NAME);

        aChoiceSecurity = MHUtil::Allocate (aVectorSecurity->Size() + 1, 31);
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            STRNCPY (aChoiceSecurity[f], s->GetName()->Get(), 30)
        }
    }
}



/**
*  Create the window key to get or set chart type value for window area
*/
void PrefPortfolioChartData::setWindowKey (int row, int type) {
    if (type == 0) {
        if (row == 1)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_TOP);
        else if (row == 2)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_TOP2);
        else if (row == 3)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_MIDDLE);
        else if (row == 4)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_MIDDLE2);
        else if (row == 5)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_BOTTOM);
        else if (row == 6)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_BOTTOM2);
    }
    else {
        if (row == 1)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_TOP));
        else if (row == 2)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_TOP2));
        else if (row == 3)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_MIDDLE));
        else if (row == 4)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_MIDDLE2));
        else if (row == 5)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_BOTTOM));
        else if (row == 6)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_BOTTOM2));
    }
}



/**
*
*/
bool PrefPortfolioChartData::value (const char* data, int row, int col) {
    if (row == 0) {
        for (int f = 0; f <= XTRADER_LAST_LAYOUT; f++) {
            if (strcmp (data, aChoiceLayout[f]) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_LAYOUT), aView, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT, f);
                break;
            }
        }
    }
    else if (row >= 1 && row <= 6) {
        setWindowKey (row, 0);

        for (int f = 0; f < aModels; f++) {
            if (strcmp (data, aChoiceModels[f]) == 0) {
                DBFactory::SetPrefIntValue (aBuffer, 0, aModels - 1, 1, f);
                break;
            }
        }
    }
    else if (row == 7)
        DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULTP_LINE_WIDTH, atoi (data));
    else if (row == 8)
        DBFactory::SetPrefIntValue (STRING(PREF_BGPCOLOR), aView, 0, 255, 0, atoi (data));
    else if (row == 9) {
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];

            if (strcmp (s->GetName()->Get(), data) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0, s->GetID());
                break;
            }
        }
    }

    return true;
}



/**
*
*/
const char* PrefPortfolioChartData::value (int row, int col) {
    *aBuffer = '\0';

    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(PROPERTY);

            case 1:
                return STRING(VALUE);
        }
    }
    else {
        if (row == 0) {
            if (col == 0)
                return STRING(CHART_LAYOUT);
            else {
                SNPRINTF (aBuffer, 99, "%s_%d", STRING(PREF_CHARTP_LAYOUT), aView);
                return aChoiceLayout[DBFactory::GetPrefIntValue (aBuffer, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT)];
            }
        }
        else if (row >= 1 && row <= 6) {
            if (col == 0) {
                setWindowKey (row, 1);
                return aBuffer;
            }
            else {
                setWindowKey (row, 0);
                return aChoiceModels[DBFactory::GetPrefIntValue (aBuffer, 0, aModels - 1, 0)];
            }
        }
        else if (row == 7) {
            if (col == 0)
                return STRING(INFO_TICK_WIDTH);
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULTP_LINE_WIDTH));
        }
        else if (row == 8) {
            if (col == 0)
                SNPRINTF (aBuffer, 999, "%s", STRING(CANVAS_COLOR));
            else
                SNPRINTF (aBuffer, 999, "%d", DBFactory::GetPrefIntValue (STRING(PREF_BGPCOLOR), aView, 0, 255, 0));
        }
        else if (row == 9) {
            if (col == 0)
                return STRING(RELATIVE_);
            else {
                int sec = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0);

                for (int f = 0; f < aVectorSecurity->Size(); f++) {
                    Security* s = (Security*) (*aVectorSecurity)[f];
                    if (s->GetID() == sec) return aChoiceSecurity[f];
                }
                return "";
            }
        }
    }

    return aBuffer;
}



/**
*
*/
int PrefPortfolioChartData::width (int col) {
    return 205;
}
