/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/PrefPriceChartData.h>
#include <db/DBFactory.h>
#include <db/Pref.h>
#include <db/Security.h>
#include <chart/PriceChart.h>
#include <Enums.h>
#include <Resource.h>
#include <MHPoint.h>
#include <FL/Fl.h>
#include <math.h>



/**
*  Choices for price are:
*  Auto reload of chart when price has changed
*  Chart Area Layout
*  Axis Layout
*  Chart top left
*  Chart top right
*  Chart middle left
*  Chart middle right
*  Chart bottom left
*  Chart bottom right
*/
PrefPriceChartData::PrefPriceChartData (int view) :
BaseData () {
    int f;

    PriceChart   p (0, -1, MHDate::WEEKDAY);
    ReportTypes* types = p.Choice ();

    aView           = view;
    aModels         = types->aNames.Size();
    aChoiceLayout   = MHUtil::Allocate (5, 40);
    aChoiceModels   = MHUtil::Allocate (aModels, 30);
    aChoiceSecurity = 0;
    aVectorSecurity = new MHVector ();

    STRNCPY (aChoiceLayout[0], STRING(LAYOUT_1), 29)
    STRNCPY (aChoiceLayout[1], STRING(LAYOUT_2EQUAL), 29)
    STRNCPY (aChoiceLayout[2], STRING(LAYOUT_2), 29)
    STRNCPY (aChoiceLayout[3], STRING(LAYOUT_3EQUAL), 29)
    STRNCPY (aChoiceLayout[4], STRING(LAYOUT_3), 29)

    for (f = 0; f < aModels; f++) {
        MHString* s = (MHString*) types->aNames[f];
        STRNCPY (aChoiceModels[f], s->Get(), 29)
    }
    reload ("Security");
}



/**
*
*/
PrefPriceChartData::~PrefPriceChartData () {
    MHUtil::Delete (aChoiceLayout, 5);
    MHUtil::Delete (aChoiceModels,  aModels);
    if (aChoiceSecurity) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);

    aVectorSecurity->Erase ();
    delete aVectorSecurity;
}



/**
*
*/
Fl_Align PrefPriceChartData::align (int row, int col) {
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** PrefPriceChartData::choice (int row, int col, int& size, int& selected, char* label) {
    size = selected = 0;

    if (row == 0) {
        SNPRINTF (aBuffer, 99, "%s_%d", STRING(PREF_CHART_LAYOUT), aView);
        size     = 5;
        selected = DBFactory::GetPrefIntValue (aBuffer, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT);
        return (const char**) aChoiceLayout;
    }
    else if (row >= 1 && row <= 6) {
        setWindowKey (row, 0);
        size     = aModels;
        selected = DBFactory::GetPrefIntValue (aBuffer, 0, aModels, 0);
        return (const char**) aChoiceModels;
    }
    else if (row == 14) {
        int sec = DBFactory::GetPrefIntValue (STRING(PREF_CHART_RELATIVE), aView, 0, 999999999, 0);

        size     = aVectorSecurity->Size();
        selected = 0;

        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            if (s->GetID() == sec) {
                selected = f;
                break;
            }
        }
        strncpy (label, STRING(SELECT_SECURITY), 99);
        return (const char**) aChoiceSecurity;
    }
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR PrefPriceChartData::editor_type (int row, int col, bool force_custom) {
    if (col == 1) {
        switch (row) {
            case 7:
            case 9:
            case 10:
            case 11:
            case 12:
            case 15:
            case 16:
            case 17:
            case 18:
                return FL_INTEGER_EDITOR;

            case 8:
                return FL_DLG_COLOR_EDITOR;

            case 14:
                return FL_DLG_LIST_EDITOR;

            default:
                return FL_LIST_EDITOR;
        }
    }
    return (FL_TABLE_EDITOR)0;
}



/**
*
*/
void PrefPriceChartData::reload (const char* message) {
    if (strcmp (message, "Security") == 0) {
        if (aChoiceSecurity) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);
        aVectorSecurity->Erase ();
        Security::Load (aVectorSecurity);
        aVectorSecurity->Sort (Security::SORT_NAME);

        aChoiceSecurity = MHUtil::Allocate (aVectorSecurity->Size() + 1, 31);
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            STRNCPY (aChoiceSecurity[f], s->GetName()->Get(), 30)
        }
    }
}



/**
*  Create the window key to get or set chart type value for window area
*/
void PrefPriceChartData::setWindowKey (int row, int type) {
    if (type == 0) {
        if (row == 1)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_TOP);
        else if (row == 2)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_TOP2);
        else if (row == 3)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_MIDDLE);
        else if (row == 4)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_MIDDLE2);
        else if (row == 5)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_BOTTOM);
        else if (row == 6)
            SNPRINTF (aBuffer, 99, "%s_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_BOTTOM2);
    }
    else {
        if (row == 1)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_TOP));
        else if (row == 2)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_TOP2));
        else if (row == 3)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_MIDDLE));
        else if (row == 4)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_MIDDLE2));
        else if (row == 5)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_BOTTOM));
        else if (row == 6)
            SNPRINTF (aBuffer, 999, "%s", STRING(CHART_AREA_BOTTOM2));
    }
}



/**
*
*/
bool PrefPriceChartData::value (const char* data, int row, int col) {
    if (row == 0) {
        for (int f = 0; f <= XTRADER_LAST_LAYOUT; f++) {
            if (strcmp (data, aChoiceLayout[f]) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_CHART_LAYOUT), aView, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT, f);
                break;
            }
        }
    }
    else if (row >= 1 && row <= 6) {
        setWindowKey (row, 0);

        for (int f = 0; f < aModels; f++) {
            if (strcmp (data, aChoiceModels[f]) == 0) {
                DBFactory::SetPrefIntValue (aBuffer, 0, aModels - 1, 1, f);
                break;
            }
        }
    }
    else if (row == 7)
        DBFactory::SetPrefIntValue (STRING(PREF_CHART_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULT_LINE_WIDTH, atoi (data));
    else if (row == 8)
        DBFactory::SetPrefIntValue (STRING(PREF_BGCOLOR), aView, 0, 255, 0, atoi (data));
    else if (row == 9)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_SHORT), aView, 3, 100, MHPoint::MVG_SHORT, atoi (data));
    else if (row == 10)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_LONG), aView, 3, 100, MHPoint::MVG_LONG, atoi (data));
    else if (row == 11)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_ATR), aView, 3, 100, MHPoint::MVG_ATR, atoi (data));
    else if (row == 12)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_BOLLINGER), aView, 3, 100, MHPoint::MVG_BOLLINGER, atoi (data));
    else if (row == 13)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_MOMENTUM), aView, 3, 100, MHPoint::MVG_MOMENTUM, atoi (data));
    else if (row == 14) {
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];

            if (strcmp (s->GetName()->Get(), data) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_CHART_RELATIVE), aView, 0, 999999999, 0, s->GetID());
                break;
            }
        }
    }
    else if (row == 15)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_RSI), aView, 3, 100, MHPoint::MVG_RSI, atoi (data));
    else if (row == 16)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_STDDEV), aView, 3, 100, MHPoint::MVG_STD, atoi (data));
    else if (row == 17)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_STOCHASTICK), aView, 3, 100, MHPoint::MVG_STOCHASTICS_K, atoi (data));
    else if (row == 18)
        DBFactory::SetPrefIntValue (STRING(PREF_MVG_STOCHASTICD), aView, 3, 100, MHPoint::MVG_STOCHASTICS_D, atoi (data));

    return true;
}



/**
*
*/
const char* PrefPriceChartData::value (int row, int col) {
    *aBuffer = '\0';

    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(PROPERTY);

            case 1:
                return STRING(VALUE);
        }
    }
    else {
        if (row == 0) {
            if (col == 0)
                return STRING(CHART_LAYOUT);
            else {
                SNPRINTF (aBuffer, 99, "%s_%d", STRING(PREF_CHART_LAYOUT), aView);
                return aChoiceLayout[DBFactory::GetPrefIntValue (aBuffer, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT)];
            }
        }
        else if (row >= 1 && row <= 6) {
            if (col == 0) {
                setWindowKey (row, 1);
                return aBuffer;
            }
            else {
                setWindowKey (row, 0);
                return aChoiceModels[DBFactory::GetPrefIntValue (aBuffer, 0, aModels - 1, 0)];
            }
        }
        else if (row == 7) {
            if (col == 0)
                return STRING(INFO_TICK_WIDTH);
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHART_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULT_LINE_WIDTH));
        }
        else if (row == 8) {
            if (col == 0)
                SNPRINTF (aBuffer, 999, "%s", STRING(CANVAS_COLOR));
            else
                SNPRINTF (aBuffer, 999, "%d", DBFactory::GetPrefIntValue (STRING(PREF_BGCOLOR), aView, 0, 255, 0));
        }
        else if (row == 9) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s 1 (3 - 100 %s)", STRING(MOVING_AVERAGE), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_SHORT), aView, 3, 100, MHPoint::MVG_SHORT));
        }
        else if (row == 10) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s 2 (3 - 100 %s)", STRING(MOVING_AVERAGE), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_LONG), aView, 3, 100, MHPoint::MVG_LONG));
        }
        else if (row == 11) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s (3 - 100 %s)", STRING(ATR), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_ATR), aView, 3, 100, MHPoint::MVG_ATR));
        }
        else if (row == 12) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s (3 - 100 %s)", STRING(BOLLINGER), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_BOLLINGER), aView, 3, 100, MHPoint::MVG_BOLLINGER));
        }
        else if (row == 13) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s (3 - 100 %s)", STRING(MOMENTUM), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_MOMENTUM), aView, 3, 100, MHPoint::MVG_MOMENTUM));
        }
        else if (row == 14) {
            if (col == 0)
                return STRING(RELATIVE_);
            else {
                int sec = DBFactory::GetPrefIntValue (STRING(PREF_CHART_RELATIVE), aView, 0, 999999999, 0);

                for (int f = 0; f < aVectorSecurity->Size(); f++) {
                    Security* s = (Security*) (*aVectorSecurity)[f];
                    if (s->GetID() == sec) return aChoiceSecurity[f];
                }
                return "";
            }
        }
        else if (row == 15) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s (3 - 100 %s)", STRING(RSI), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_RSI), aView, 3, 100, MHPoint::MVG_RSI));
        }
        else if (row == 16) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s (3 - 100 %s)", STRING(STDDEV), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_STDDEV), aView, 3, 100, MHPoint::MVG_STD));
        }
        else if (row == 17) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s (3 - 100 %s)", STRING(STOCHASTICS), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_STOCHASTICK), aView, 3, 100, MHPoint::MVG_STOCHASTICS_K));
        }
        else if (row == 18) {
            if (col == 0)
                SNPRINTF (aBuffer, 99, "%s %s (3 - 100 %s)", STRING(STOCHASTICS), STRING(MVG), STRING(DAYS));
            else
                SNPRINTF (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_MVG_STOCHASTICD), aView, 3, 100, MHPoint::MVG_STOCHASTICS_D));
        }
    }

    return aBuffer;
}



/**
*
*/
int PrefPriceChartData::width (int col) {
    return 205;
}
